@@ -622,6 +622,92 @@ class TestTEIConversions:
                 else:
                     print("No offset differences detected between conversion and expected output")
 
+    def test_formula_extraction_in_json(self):
+        """Test that formula elements are extracted as passages in JSON conversion."""
+        from grobid_client.format.TEI2LossyJSON import TEI2LossyJSONConverter
+
+        # Use the actual TEI file from test resources which contains formulas
+        tei_file = os.path.join(TEST_DATA_PATH, '0046d83a-edd6-4631-b57c-755cdcce8b7f.tei.xml')
+
+        converter = TEI2LossyJSONConverter()
+        json_data = converter.convert_tei_file(tei_file, stream=False)
+
+        # Find formula passages
+        body_text = json_data.get('body_text', [])
+        formula_passages = [p for p in body_text if p.get('type') == 'formula']
+
+        # The test file contains 2 formulas
+        assert len(formula_passages) >= 2, "Should extract at least 2 formulas from test file"
+
+        # Check formula structure
+        for formula in formula_passages:
+            assert 'text' in formula, "Formula should have text"
+            assert 'id' in formula, "Formula should have id"
+            assert formula['text'].strip(), "Formula text should not be empty"
+            
+            # The test formulas have labels
+            if 'label' in formula:
+                assert formula['label'].strip(), "Formula label should not be empty"
+
+        # Check for specific formula content from test file
+        formula_texts = [f.get('text', '') for f in formula_passages]
+        assert any('Fext' in t for t in formula_texts), "Should extract formula containing 'Fext'"
+
+    def test_formula_extraction_in_markdown(self):
+        """Test that formula elements are included in Markdown conversion."""
+        from grobid_client.format.TEI2Markdown import TEI2MarkdownConverter
+
+        # Use the actual TEI file from test resources which contains formulas
+        tei_file = os.path.join(TEST_DATA_PATH, '0046d83a-edd6-4631-b57c-755cdcce8b7f.tei.xml')
+
+        converter = TEI2MarkdownConverter()
+        markdown = converter.convert_tei_file(tei_file)
+
+        # Check that formula content is present
+        assert 'Fext' in markdown, "Markdown should contain formula text 'Fext'"
+        
+        # Check that formula is italicized (surrounded by asterisks)
+        assert '*Fext' in markdown, "Formula should be italicized in Markdown"
+
+    def test_header_only_div_in_json(self):
+        """Test that headers without paragraphs are included in JSON conversion."""
+        from grobid_client.format.TEI2LossyJSON import TEI2LossyJSONConverter
+
+        # Use the actual TEI file from test resources
+        tei_file = os.path.join(TEST_DATA_PATH, '0046d83a-edd6-4631-b57c-755cdcce8b7f.tei.xml')
+
+        converter = TEI2LossyJSONConverter()
+        json_data = converter.convert_tei_file(tei_file, stream=False)
+
+        # Collect all section headers from the body_text
+        body_text = json_data.get('body_text', [])
+        section_headers = set()
+        for passage in body_text:
+            if 'head_section' in passage and passage['head_section']:
+                section_headers.add(passage['head_section'])
+
+        # Check that common sections are present
+        # The test file has Acknowledgements and Competing interests headers
+        assert 'Competing interests' in section_headers, "Should include 'Competing interests' header"
+        
+        # Verify we have a good number of sections
+        assert len(section_headers) >= 10, f"Should extract many section headers, got {len(section_headers)}"
+
+    def test_header_only_div_in_markdown(self):
+        """Test that headers without paragraphs are included in Markdown conversion."""
+        from grobid_client.format.TEI2Markdown import TEI2MarkdownConverter
+
+        # Use the actual TEI file from test resources
+        tei_file = os.path.join(TEST_DATA_PATH, '0046d83a-edd6-4631-b57c-755cdcce8b7f.tei.xml')
+
+        converter = TEI2MarkdownConverter()
+        markdown = converter.convert_tei_file(tei_file)
+
+        # Check that the Acknowledgements header is present
+        assert 'Acknowledgements' in markdown, "Markdown should contain 'Acknowledgements' header"
+        
+        # Check it's formatted as a header (preceded by ###)
+        assert '### Acknowledgements' in markdown, "Acknowledgements should be formatted as Markdown header"
 
     def test_conversion_JSON(self):
         from grobid_client.format.TEI2LossyJSON import TEI2LossyJSONConverter
@@ -648,3 +734,218 @@ class TestTEIConversions:
                         actual_text = paragraph_text[offset_start:offset_end]
                         assert actual_text == ref_text, f"Reference text at offsets ({offset_start}-{offset_end}) should match '{ref_text}' but got '{actual_text}'"
 
+    def test_header_extraction_from_mjb_file(self):
+        """Test specific header extraction for mjb3wlzxcb2mc-migowebupload-1766042162782.grobid.tei.xml"""
+        from grobid_client.format.TEI2LossyJSON import TEI2LossyJSONConverter
+        
+        # Calculate path to the specific test file in resources/test_pdf
+        project_root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
+        tei_file = os.path.join(project_root, 'resources', 'test_pdf', 'mjb3wlzxcb2mc-migowebupload-1766042162782.grobid.tei.xml')
+        
+        converter = TEI2LossyJSONConverter()
+        json_data = converter.convert_tei_file(tei_file, stream=False)
+        
+        body_text = json_data.get('body_text', [])
+        
+        # Verify 'Methods' is identified
+        methods_found = False
+        for passage in body_text:
+            # Based on debug output, it appears as head_paragraph
+            if passage.get('head_paragraph') == 'Methods' or passage.get('head_section') == 'Methods':
+                methods_found = True
+                break
+        
+        assert methods_found, "'Methods' header should be extracted as head_paragraph or head_section"
+
+        # Verify other sections are present as head_section
+        sections_found = set()
+        for passage in body_text:
+            if 'head_section' in passage:
+                sections_found.add(passage['head_section'])
+                
+        expected_sections = [
+            "Study Design, Patients, and Dosing",
+            "Study Assessments and Endpoints",
+            "Statistical Analyses"
+        ]
+        
+        for section in expected_sections:
+            assert section in sections_found, f"'{section}' should be in extracted sections"
+
+    def test_withdrawn_article_json_conversion(self):
+        """Test JSON conversion for a withdrawn/empty article TEI file."""
+        from grobid_client.format.TEI2LossyJSON import TEI2LossyJSONConverter
+
+        # Use the withdrawn article TEI file from test resources
+        tei_file = os.path.join(TEST_DATA_PATH, 'article_withdrawn.grobid.tei.xml')
+
+        # Verify the test TEI file exists
+        assert os.path.exists(tei_file), f"Test TEI file should exist at {tei_file}"
+
+        converter = TEI2LossyJSONConverter()
+        json_data = converter.convert_tei_file(tei_file, stream=False)
+
+        # The converter should return a non-None result (not fail) for valid but empty TEI
+        assert json_data is not None, "Withdrawn/empty TEI should return non-None JSON result"
+        assert isinstance(json_data, dict), "Result should be a dictionary"
+
+        # Check basic structure is present
+        assert 'biblio' in json_data, "Should have biblio section"
+        assert 'body_text' in json_data, "Should have body_text section"
+
+    def test_withdrawn_article_markdown_conversion(self):
+        """Test Markdown conversion for a withdrawn/empty article TEI file."""
+        from grobid_client.format.TEI2Markdown import TEI2MarkdownConverter
+
+        # Use the withdrawn article TEI file from test resources
+        tei_file = os.path.join(TEST_DATA_PATH, 'article_withdrawn.grobid.tei.xml')
+
+        # Verify the test TEI file exists
+        assert os.path.exists(tei_file), f"Test TEI file should exist at {tei_file}"
+
+        converter = TEI2MarkdownConverter()
+        markdown_data = converter.convert_tei_file(tei_file)
+
+        # The converter should return a non-None result (not fail) for valid but empty TEI
+        # It may return an empty string, but should not return None
+        assert markdown_data is not None, "Withdrawn/empty TEI should return non-None Markdown result"
+        assert isinstance(markdown_data, str), "Result should be a string"
+
+    def test_json_conversion_stream_mode_with_real_file(self):
+        """Test JSON conversion in streaming mode with a real TEI file."""
+        from grobid_client.format.TEI2LossyJSON import TEI2LossyJSONConverter
+
+        # Use the actual TEI file from test resources
+        tei_file = os.path.join(TEST_DATA_PATH, '0046d83a-edd6-4631-b57c-755cdcce8b7f.tei.xml')
+
+        # Verify the test TEI file exists
+        assert os.path.exists(tei_file), f"Test TEI file should exist at {tei_file}"
+
+        converter = TEI2LossyJSONConverter()
+        passages_generator = converter.convert_tei_file(tei_file, stream=True)
+
+        # Should return a generator/iterator, not None
+        assert passages_generator is not None, "Streaming mode should return a generator"
+
+        # Collect all passages from the generator
+        passages = list(passages_generator)
+
+        # Should have extracted some passages
+        assert len(passages) > 0, "Should extract at least one passage in streaming mode"
+
+        # Each passage should be a dict with expected structure
+        for passage in passages:
+            assert isinstance(passage, dict), "Each passage should be a dictionary"
+            assert 'id' in passage, "Passage should have an id"
+            assert 'text' in passage, "Passage should have text"
+
+    def test_json_conversion_stream_mode_with_empty_tei(self):
+        """Test JSON conversion in streaming mode with empty TEI content."""
+        from grobid_client.format.TEI2LossyJSON import TEI2LossyJSONConverter
+
+        # Test with empty TEI content
+        empty_tei = """<?xml version="1.0" encoding="UTF-8"?>
+<TEI xmlns="http://www.tei-c.org/ns/1.0">
+</TEI>"""
+
+        # Create a temporary TEI file with empty content
+        with tempfile.NamedTemporaryFile(mode='w', suffix='.tei.xml', delete=False) as tei_file:
+            tei_file.write(empty_tei)
+            tei_path = tei_file.name
+
+        try:
+            converter = TEI2LossyJSONConverter()
+            passages_generator = converter.convert_tei_file(tei_path, stream=True)
+
+            # Should return an empty iterator for empty TEI, not None
+            assert passages_generator is not None, "Streaming mode should return an iterator even for empty TEI"
+
+            # Collect all passages - should be empty for empty TEI
+            passages = list(passages_generator)
+
+            # Empty TEI should produce no passages
+            assert isinstance(passages, list), "Result should be convertible to list"
+
+        finally:
+            # Clean up temporary file
+            os.unlink(tei_path)
+
+    def test_json_conversion_stream_mode_with_withdrawn_article(self):
+        """Test JSON conversion in streaming mode with withdrawn/empty article."""
+        from grobid_client.format.TEI2LossyJSON import TEI2LossyJSONConverter
+
+        # Use the withdrawn article TEI file from test resources
+        tei_file = os.path.join(TEST_DATA_PATH, 'article_withdrawn.grobid.tei.xml')
+
+        # Verify the test TEI file exists
+        assert os.path.exists(tei_file), f"Test TEI file should exist at {tei_file}"
+
+        converter = TEI2LossyJSONConverter()
+        passages_generator = converter.convert_tei_file(tei_file, stream=True)
+
+        # Should return a generator/iterator, not None
+        assert passages_generator is not None, "Streaming mode should return an iterator for withdrawn article"
+
+        # Collect all passages - may be empty for withdrawn article
+        passages = list(passages_generator)
+
+        # Should be a list (possibly empty)
+        assert isinstance(passages, list), "Result should be convertible to list"
+
+    def test_json_conversion_stream_mode_validates_refs(self):
+        """Test that streaming mode validates reference offsets correctly."""
+        from grobid_client.format.TEI2LossyJSON import TEI2LossyJSONConverter
+
+        # Use file with references
+        tei_file = os.path.join(TEST_DATA_PATH, '0046d83a-edd6-4631-b57c-755cdcce8b7f.tei.xml')
+
+        converter = TEI2LossyJSONConverter(validate_refs=True)
+        passages_generator = converter.convert_tei_file(tei_file, stream=True)
+
+        # Collect all passages - this should not raise assertion errors if refs are valid
+        passages = list(passages_generator)
+
+        # Check passages with refs have valid offsets
+        for passage in passages:
+            if 'refs' in passage and passage['refs']:
+                for ref in passage['refs']:
+                    offset_start = ref.get('offset_start', -1)
+                    offset_end = ref.get('offset_end', -1)
+                    ref_text = ref.get('text', '')
+                    passage_text = passage.get('text', '')
+
+                    # Validate offset bounds
+                    assert 0 <= offset_start < offset_end <= len(passage_text), \
+                        f"Invalid ref offsets: {offset_start}-{offset_end} for text length {len(passage_text)}"
+
+                    # Validate text matches
+                    actual_text = passage_text[offset_start:offset_end]
+                    assert actual_text == ref_text, \
+                        f"Ref text mismatch: expected '{ref_text}', got '{actual_text}'"
+
+    def test_json_conversion_stream_vs_non_stream_consistency(self):
+        """Test that streaming and non-streaming modes produce consistent results."""
+        from grobid_client.format.TEI2LossyJSON import TEI2LossyJSONConverter
+
+        # Use the actual TEI file from test resources
+        tei_file = os.path.join(TEST_DATA_PATH, '0046d83a-edd6-4631-b57c-755cdcce8b7f.tei.xml')
+
+        converter = TEI2LossyJSONConverter()
+
+        # Get non-streaming result
+        non_stream_result = converter.convert_tei_file(tei_file, stream=False)
+        body_text_non_stream = non_stream_result.get('body_text', [])
+
+        # Get streaming result
+        stream_result = converter.convert_tei_file(tei_file, stream=True)
+        body_text_stream = list(stream_result)
+
+        # Both should have the same number of passages
+        assert len(body_text_non_stream) == len(body_text_stream), \
+            f"Stream and non-stream should have same number of passages: {len(body_text_stream)} vs {len(body_text_non_stream)}"
+
+        # Compare passage texts
+        for i, (stream_p, non_stream_p) in enumerate(zip(body_text_stream, body_text_non_stream)):
+            assert stream_p.get('text') == non_stream_p.get('text'), \
+                f"Passage {i} text mismatch between stream and non-stream modes"
+
